/**
 * Copyright 2001-2003 The eXo Platform SARL         All rights reserved.
 * Please look at license.txt in info directory for more license detail.
 **/

/**
 * Created by The eXo Platform SARL
 * Author : Daniel Summer
 *          danny_xcz@users.sourceforge.net
 * Date: 25/5/2004
 */
package org.exoplatform.services.ldap;

import netscape.ldap.LDAPAttribute;
import netscape.ldap.LDAPAttributeSet;
import netscape.ldap.LDAPEntry;
import netscape.ldap.LDAPSearchResults;
import netscape.ldap.LDAPv2;
import org.exoplatform.container.PortalContainer;
import org.exoplatform.services.log.LogService;
import org.exoplatform.test.BasicTestCase;

public class TestLDAPService extends BasicTestCase {

	private LDAPServiceContainer ldapServiceContainer;
	private LDAPService ldapService;
	private LogService logService;

	public TestLDAPService(String s) {
		super(s);
	}

	protected String getDescription() {
		return "test ldap service";
	}

	public void setUp() throws Exception {

		if (ldapServiceContainer == null) {
			PortalContainer pcontainer = PortalContainer.getInstance();
			ldapServiceContainer =
				(LDAPServiceContainer) pcontainer.getComponentInstanceOfType(
					LDAPServiceContainer.class);
			//assertNotNull(ldapServiceContainer);
			ldapService = ldapServiceContainer.createLDAPService();
		}
	}

	public void testLDAPAddEntry() throws Exception {

		String objectclasses[] = { "exouser" };
		LDAPAttribute attr1 = new LDAPAttribute("objectclass", objectclasses);
		LDAPAttribute attr2 = new LDAPAttribute("uid", "danny_xcz");
		LDAPAttribute attr3 = new LDAPAttribute("userPassword", "pass");
		LDAPAttributeSet attrSet = new LDAPAttributeSet();
		attrSet.add(attr1);
		attrSet.add(attr2);
		attrSet.add(attr3);
		String dn = "uid=danny_xcz,ou=users,dc=exoplatform,dc=com";
		LDAPEntry newEntry = new LDAPEntry(dn, attrSet);
		ldapService.add(newEntry);
	}

	public void testLDAPSearch() throws Exception {
		/* Specify the search criteria. */
		String baseDN = "dc=exoplatform,dc=com";
		int searchScope = LDAPv2.SCOPE_SUB;
		String searchFilter = "(uid=danny_xcz)";
		String getAttrs[] = { "userPassword", "mail" };
		/* Send the search request. */
		LDAPSearchResults res =
			ldapService.search(
				baseDN,
				searchScope,
				searchFilter,
				getAttrs,
				false);

		LDAPEntry nextEntry = res.next();
		LDAPAttribute passwordAttr = nextEntry.getAttribute("userPassword");
		assertEquals(1, passwordAttr.size());
	}

}
