/***************************************************************************
 * Copyright 2001-2003 The eXo Platform SARL         All rights reserved.  *
 * Please look at license.txt in info directory for more license detail.   *
 **************************************************************************/
package org.exoplatform.services.organization.ldap;
import java.io.InputStream;
import java.io.OutputStream;
import java.sql.Timestamp;
import java.util.Collection;
import java.util.Enumeration;
import java.util.Iterator;
import java.util.jar.JarEntry;
import java.util.jar.JarFile;
import java.util.jar.JarOutputStream;

import net.sf.hibernate.Session;

import org.exoplatform.commons.utils.IOUtil;
import org.exoplatform.services.backup.ImporterExporter;
import org.exoplatform.services.ldap.LDAPService;
import org.exoplatform.services.ldap.LDAPServiceContainer;
import org.exoplatform.services.organization.impl.GroupImpl;
import org.exoplatform.services.organization.impl.MembershipImpl;
import org.exoplatform.services.organization.impl.MembershipTypeImpl;
import org.exoplatform.services.organization.impl.UserBackupData;
import org.exoplatform.services.organization.impl.UserImpl;
import org.exoplatform.services.organization.impl.UserProfileImpl;

import com.thoughtworks.xstream.XStream;
import com.thoughtworks.xstream.io.xml.XppDriver;
/**
 * May 27, 2004 
 * @author: Tuan Nguyen
 * @email:   tuan08@users.sourceforge.net
 * @version: $ID$
 **/
public class LDAPOrganizationImporterExporter implements ImporterExporter {
	private XStream xstream_;
	private OrganizationServiceLDAPImpl service_;
	private LDAPServiceContainer ldapServiceContainer_;
	public LDAPOrganizationImporterExporter(
		OrganizationServiceLDAPImpl service,
		LDAPServiceContainer ldapServiceContainer) {
		ldapServiceContainer_ = ldapServiceContainer;
		service_ = service;
		xstream_ = new XStream(new XppDriver());
		xstream_.alias("userData", UserBackupData.class);
		xstream_.alias("membership", MembershipImpl.class);
		xstream_.alias("membershipType", MembershipTypeImpl.class);
		xstream_.alias("group", GroupImpl.class);
		xstream_.alias("date", Timestamp.class);
	}
	public String getName() {
		return "organization";
	}
	public String getDescription() {
		return "Export User , UserProfile , Memberships of an user";
	}
	public void exportUserData(
		String userName,
		Object transaction,
		OutputStream out,
		JarEntry entry)
		throws Exception {
		Session session = (Session) transaction;
		LDAPService service = ldapServiceContainer_.createLDAPService();
		UserImpl user =
			(UserImpl) service_.userLDAPHandler_.findUserByName(
				userName,
				service);
		Collection memberships =
			service_.membershipLDAPHandler_.findMembershipsByUser(
				userName,
				session);
		UserProfileImpl up =
			(
				UserProfileImpl) service_
					.userProfileLDAPHandler_
					.findUserProfileByName(
				userName,
				service);
		UserBackupData data = new UserBackupData(user, up, memberships);
		String xml = xstream_.toXML(data);
		out.write(xml.getBytes());
	}
	public void importUserData(
		String userName,
		Object transaction,
		InputStream is,
		JarEntry entry)
		throws Exception {
		Session session = (Session) transaction;
		removeExistingUserData(userName, session);
		String xml = IOUtil.getStreamContentAsString(is);
		UserBackupData data = (UserBackupData) xstream_.fromXML(xml);
		UserImpl user = data.getUser();
		UserProfileImpl up = data.getUserProfile();
		Collection memberships = data.getMemberships();
		service_.userLDAPHandler_.createUserEntry(user, session);
		service_.membershipLDAPHandler_.createMembershipEntries(memberships,session);
		service_.userProfileLDAPHandler_.createUserProfileEntry(up, session);
	}
  
	public void exportServiceData(Object transaction, JarOutputStream out)
		throws Exception {
		Session session = (Session) transaction;
		Collection groups =
			session.find(
				"from g in class org.exoplatform.services.organization.impl.GroupBackupData");
		Iterator i = groups.iterator();
		while (i.hasNext()) {
			GroupImpl data = (GroupImpl) i.next();
			JarEntry entry =
				new JarEntry("group/" + data.getGroupName() + ".xml");
			out.putNextEntry(entry);
			String xml = xstream_.toXML(data);
			out.write(xml.getBytes());
			out.closeEntry();
		}
		Collection membershipTypes = service_.findMembershipTypes();
		i = membershipTypes.iterator();
		while (i.hasNext()) {
			MembershipTypeImpl data = (MembershipTypeImpl) i.next();
			JarEntry entry =
				new JarEntry("membershipType/" + data.getName() + ".xml");
			out.putNextEntry(entry);
			String xml = xstream_.toXML(data);
			out.write(xml.getBytes());
			out.closeEntry();
		}
	}
	public void importServiceData(Object transaction, JarFile jar)
		throws Exception {
		Session session = (Session) transaction;
		LDAPService service = ldapServiceContainer_.createLDAPService();
		Enumeration e = jar.entries();
		while (e.hasMoreElements()) {
			JarEntry entry = (JarEntry) e.nextElement();
			InputStream is = jar.getInputStream(entry);
			String xml = IOUtil.getStreamContentAsString(is);
			String entryName = entry.getName();
			if (entryName.startsWith("group")) {
				GroupImpl data = (GroupImpl) xstream_.fromXML(xml);
				GroupLDAPHandler.removeGroupEntry(data.getGroupName(), session,service);
				session.save(data);
			} else if (entryName.startsWith("membershipType")) {
				MembershipTypeImpl data =
					(MembershipTypeImpl) xstream_.fromXML(xml);
				MembershipTypeLDAPHandler.removeMembershipTypeEntry(
					data.getName(),
					session,service);
				session.save(data);
			}
		}
	}
	private void removeExistingUserData(String userName, Session session)
		throws Exception {
			LDAPService service = ldapServiceContainer_.createLDAPService();
		UserImpl user =
			(UserImpl) service_.userLDAPHandler_.findUserByName(
				userName,
			service);
		if (user != null) {
			service_.userLDAPHandler_.removeUserEntry(user, session);
			MembershipLDAPHandler.removeMembershipEntriesOfUser(userName,session,service);
			UserProfileLDAPHandler.removeUserProfileEntry(userName, session,service);
		}
	}
}