/**
 * Copyright 2001-2004 The eXo Platform SARL         All rights reserved.
 * Please look at license.txt in info directory for more license detail.
 **/
package org.exoplatform.services.organization.ldap;

import java.util.ArrayList;
import java.util.Collection;
import java.util.Iterator;
import java.util.List;
import java.util.Vector;

import net.sf.hibernate.Session;
import netscape.ldap.LDAPAttribute;
import netscape.ldap.LDAPConnection;
import netscape.ldap.LDAPEntry;
import netscape.ldap.LDAPModification;
import netscape.ldap.LDAPModificationSet;
import netscape.ldap.LDAPSearchResults;

import org.apache.commons.lang.StringUtils;
import org.exoplatform.services.database.HibernateService;
import org.exoplatform.services.database.XResources;
import org.exoplatform.services.ldap.LDAPService;
import org.exoplatform.services.ldap.LDAPServiceContainer;
import org.exoplatform.services.organization.Group;
import org.exoplatform.services.organization.Membership;
import org.exoplatform.services.organization.MembershipEventListener;
import org.exoplatform.services.organization.User;
import org.exoplatform.services.organization.impl.MembershipImpl;
/**
 * Created by the eXo platform team
 * User: Daniel Summer
 * Date: 28 august 2004
 *
 */
public class MembershipLDAPHandler extends BaseLDAPHandler {
	private final String BASEURL = OrganizationLDAPConfig.BASE_URL;
	private final String PORTALURL = OrganizationLDAPConfig.PORTAL_URL;
	private final String ROLESURL = OrganizationLDAPConfig.ROLES_URL;
	private final String USERSURL = OrganizationLDAPConfig.USERS_URL;
	private LDAPServiceContainer serviceContainer_;
	private HibernateService hService_;
	private List listeners_;

	public MembershipLDAPHandler(
		LDAPServiceContainer serviceContainer,
		HibernateService hService) {
		serviceContainer_ = serviceContainer;
		hService_ = hService;
		listeners_ = new ArrayList(5);
	}

	public void addMembershipEventListener(MembershipEventListener listener) {
		listeners_.add(listener);
	}

	public void createMembership(Membership m) throws Exception {
		LDAPService service = null;
		Session session = null;
		try {
			service = serviceContainer_.createLDAPService();
			session = hService_.openSession();
			String MY_FILTER = "uid=" + m.getUserName();
			String MY_SEARCHBASE = USERSURL;
			LDAPSearchResults res =
				service.search(
					MY_SEARCHBASE,
					LDAPConnection.SCOPE_SUB,
					MY_FILTER,
					null,
					false);
			List l = getUtil(res);
			MembershipImpl mi = (MembershipImpl) m;
			StringBuffer b = new StringBuffer();
			b
				.append(mi.getUserName())
				.append(",")
				.append(mi.getMembershipType())
				.append(",")								
				.append(mi.getGroupId());
			LDAPEntry findEntry = (LDAPEntry) l.get(0);
			LDAPModificationSet mods = new LDAPModificationSet();
			LDAPAttribute attrRole =
				new LDAPAttribute("membership", b.toString());
			mods.add(LDAPModification.ADD, attrRole);

			preSave(m, true, session);
			service.modify(findEntry.getDN(), mods);
			postSave(m, true, session);
			session.flush();
		} finally {
			serviceContainer_.closeLDAPService(service);
			hService_.closeSession(session);
		}
	}

	public void createMembershipEntries(Collection c, Session session)
		throws Exception {
		LDAPService service = null;
		service = serviceContainer_.createLDAPService();
		Iterator i = c.iterator();
		while (i.hasNext()) {
			Membership impl = (Membership) i.next();
			String MY_FILTER = "uid=" + impl.getUserName();
			String MY_SEARCHBASE = USERSURL;

			LDAPSearchResults res =
				service.search(
					MY_SEARCHBASE,
					LDAPConnection.SCOPE_SUB,
					MY_FILTER,
					null,
					false);
			List l = getUtil(res);

			StringBuffer b = new StringBuffer();
			b
				.append(impl.getUserName())
				.append(",")
				.append(impl.getMembershipType())
				.append(",")								
				.append(impl.getGroupId());
			LDAPEntry findEntry = (LDAPEntry) l.get(0);
			LDAPModificationSet mods = new LDAPModificationSet();
			LDAPAttribute attrRole =
				new LDAPAttribute("membership", b.toString());
			mods.add(LDAPModification.ADD, attrRole);
			service.modify(findEntry.getDN(), mods);
		}
		serviceContainer_.closeLDAPService(service);
	}

	public void linkMembership(String userName, Group g, Membership m)
		throws Exception {
		Group group = null;
		User user = null;
		Session session = null;
		LDAPService service = null;

		try {
			session = hService_.openSession();
			service = serviceContainer_.createLDAPService();
			MembershipImpl impl = (MembershipImpl) m;
			String MY_FILTER = "uid=" + userName;
			String MY_SEARCHBASE = USERSURL;
			LDAPSearchResults res =
				service.search(
					MY_SEARCHBASE,
					LDAPConnection.SCOPE_SUB,
					MY_FILTER,
					null,
					false);
			List l = getUtil(res);

			boolean isNew = true;

			LDAPEntry findEntry = (LDAPEntry) l.get(0);
			StringBuffer b = new StringBuffer();
			b
				.append(userName)
				.append(",")
				.append(m.getMembershipType())
				.append(",")
				.append(g.getId());
			String membership = b.toString();

			Vector memberships =
				getutil("membership", findEntry.getAttributeSet(), false);
			for (Iterator i = memberships.iterator(); i.hasNext();) {
				String s = (String) i.next();
				if (s.equals(membership)) {
					isNew = false;
				}
			}

			if (isNew) {

				LDAPModificationSet mods = new LDAPModificationSet();
				LDAPAttribute attrMembership =
					new LDAPAttribute("membership", membership);
				mods.add(LDAPModification.ADD, attrMembership);

				preSave(m, isNew, session);
				service.modify(findEntry.getDN(), mods);
				postSave(m, isNew, session);
				session.flush();
			}
		} finally {
			hService_.closeSession(session);
			serviceContainer_.closeLDAPService(service);
		}

	}

	public void saveMembership(Membership m) throws Exception {
		LDAPService service = null;
		Session session = null;
		try {
			service = serviceContainer_.createLDAPService();
			session = hService_.openSession();
			String MY_FILTER = "membership=" + m.getId();
			String MY_SEARCHBASE = USERSURL;
			String ENTRYDN = MY_FILTER + MY_SEARCHBASE;
			LDAPSearchResults res =
				service.search(
					MY_SEARCHBASE,
					LDAPConnection.SCOPE_SUB,
					MY_FILTER,
					null,
					false);
			List l = getUtil(res);
			MembershipImpl mi = (MembershipImpl) m;
			StringBuffer b = new StringBuffer();
			b
				.append(mi.getUserName())
				.append(",")
				.append(mi.getMembershipType())
				.append(",")
				.append(mi.getGroupId());
			LDAPEntry findEntry = (LDAPEntry) l.get(0);
			String[] attrValues =
				(String[]) getutil("membership",
					findEntry.getAttributeSet(),
					false)
					.toArray();
			for (int i = 0; i < attrValues.length; i++) {
				if (mi.getId().equals(attrValues[i])) {
					attrValues[i] = b.toString();
				}
			}
			LDAPModificationSet mods = new LDAPModificationSet();
			mods.add(
				LDAPModification.REPLACE,
				new LDAPAttribute("membership", attrValues));

			preSave(m, true, session);
			service.modify(ENTRYDN, mods);
			postSave(m, true, session);
			session.flush();
		} finally {
			serviceContainer_.closeLDAPService(service);
			hService_.closeSession(session);
		}

	}

	public Membership removeMembership(String id) throws Exception {
		LDAPService service = null;
		Session session = null;
		MembershipImpl m = new MembershipImpl();
		try {
			service = serviceContainer_.createLDAPService();
			session = hService_.openSession();
			String MY_FILTER = "membership=" + m.getId();
			String MY_SEARCHBASE = USERSURL;
			String ENTRYDN = MY_FILTER + MY_SEARCHBASE;
			LDAPSearchResults res =
				service.search(
					MY_SEARCHBASE,
					LDAPConnection.SCOPE_SUB,
					MY_FILTER,
					null,
					false);
			List l = getUtil(res);
			LDAPEntry findEntry = (LDAPEntry) l.get(0);
			Vector memberships =
				getutil("membership", findEntry.getAttributeSet(), false);

			for (Iterator i = memberships.iterator(); i.hasNext();) {
				String s = (String) i.next();
				if (s.equals(id)) {
					m.setId(s);
					String[] membership = StringUtils.split(s, ",");
					m.setUserName(membership[0]);
					m.setMembershipType(membership[1]);					
					m.setGroupId(membership[2]);
					LDAPModificationSet mods = new LDAPModificationSet();
					mods.add(
						LDAPModification.DELETE,
						new LDAPAttribute("membership", id));

					preDelete(m, session);
					service.modify(ENTRYDN, mods);
					postDelete(m, session);
					session.flush();
				}
			}

		} finally {
			serviceContainer_.closeLDAPService(service);
			hService_.closeSession(session);
		}
		return m;
	}

	public Membership findMembershipByUserGroupAndType(
		String userName,
		String groupId,
		String type)
		throws Exception {
		List memberships = new ArrayList();

		LDAPService service = null;
		Session session = null;
		try {
			service = serviceContainer_.createLDAPService();
			session = hService_.openSession();
			String MY_FILTER = "uid=" + userName;
			String MY_SEARCHBASE = USERSURL;

			LDAPSearchResults res =
				service.search(
					MY_SEARCHBASE,
					LDAPConnection.SCOPE_SUB,
					MY_FILTER,
					null,
					false);

			while (res.hasMoreElements()) {
				LDAPEntry findEntry = res.next();

				String[] attrValues =
					(String[]) getutil("membership",
						findEntry.getAttributeSet(),
						false)
						.toArray();
				for (int i = 0; i < attrValues.length; i++) {
					String[] membership = StringUtils.split(attrValues[i], ",");
					if (membership[3].equals(groupId)
						&& membership[1].equals(groupId)) {
						MembershipImpl mi = new MembershipImpl();
						mi.setId(attrValues[i]);
						mi.setUserName(membership[0]);
						mi.setMembershipType(membership[1]);						
						mi.setGroupId(membership[2]);
						memberships.add(mi);
					}
				}
			}
		} finally {
			serviceContainer_.closeLDAPService(service);
			hService_.closeSession(session);
		}

		if (memberships.size() == 0) {
			return null;
		} else if (memberships.size() == 1) {
			return (Membership) memberships.get(0);
		} else {
			throw new Exception(
				"Expect 0 or 1 membership but found" + memberships.size());
		}
	}

	public Collection findMembershipsByUserAndGroup(
		String userName,
		String groupId)
		throws Exception {
		Collection memberships = new ArrayList();

		LDAPService service = null;
		Session session = null;
		try {
			service = serviceContainer_.createLDAPService();
			session = hService_.openSession();
			String MY_FILTER = "uid=" + userName;
			String MY_SEARCHBASE = USERSURL;

			LDAPSearchResults res =
				service.search(
					MY_SEARCHBASE,
					LDAPConnection.SCOPE_SUB,
					MY_FILTER,
					null,
					false);
			List l = getUtil(res);
			LDAPEntry findEntry = (LDAPEntry) l.get(0);

			Vector attrValues =
				getutil("membership", findEntry.getAttributeSet(), false);
			for (Iterator i = attrValues.iterator(); i.hasNext();) {
				String s = (String) i.next();
				String[] membership = StringUtils.split(s, ",");
				if (membership[3].equals(groupId)) {
					MembershipImpl mi = new MembershipImpl();
					mi.setId(s);
					mi.setUserName(membership[0]);
					mi.setMembershipType(membership[1]);					
					mi.setGroupId(membership[2]);
					memberships.add(mi);
				}
			}

		} finally {
			serviceContainer_.closeLDAPService(service);
			hService_.closeSession(session);
		}
		return memberships;
	}

	public Collection findMembershipsByUser(String userName) throws Exception {
		List memberships = new ArrayList();
		LDAPService service = null;
		Session session = null;
		try {
			service = serviceContainer_.createLDAPService();
			session = hService_.openSession();
			String MY_FILTER = "uid=" + userName;
			String MY_SEARCHBASE = USERSURL;

			LDAPSearchResults res =
				service.search(
					MY_SEARCHBASE,
					LDAPConnection.SCOPE_SUB,
					MY_FILTER,
					null,
					false);
			List l = getUtil(res);

			while (res.hasMoreElements()) {
				LDAPEntry findEntry = res.next();

				String[] attrValues =
					(String[]) getutil("membership",
						findEntry.getAttributeSet(),
						false)
						.toArray();
				for (int i = 0; i < attrValues.length; i++) {
					String[] membership = StringUtils.split(attrValues[i], ",");
					MembershipImpl mi = new MembershipImpl();
					mi.setId(attrValues[i]);
					mi.setUserName(membership[0]);
					mi.setMembershipType(membership[1]);					
					mi.setGroupId(membership[2]);
					memberships.add(mi);
				}
			}
		} finally {
			serviceContainer_.closeLDAPService(service);
			hService_.closeSession(session);
		}
		return memberships;
	}

	static void removeMembershipEntriesOfUser(
		String userName,
		Session session,
		LDAPService service)
		throws Exception {

		String MY_FILTER = "uid=" + userName;
		String MY_SEARCHBASE = OrganizationLDAPConfig.USERS_URL;

		LDAPSearchResults res =
			service.search(
				MY_SEARCHBASE,
				LDAPConnection.SCOPE_SUB,
				MY_FILTER,
				null,
				false);

		LDAPEntry findEntry = (LDAPEntry) res.next();
		LDAPModificationSet mods = new LDAPModificationSet();
		LDAPAttribute attrMembership = new LDAPAttribute("membership");
		mods.add(LDAPModification.DELETE, attrMembership);
		service.modify(findEntry.getDN(), mods);

	}

	static void removeMembershipEntriesOfGroup(
		Group group,
		Session session,
		LDAPService service)
		throws Exception {
		String MY_FILTER = "membership=" + "*" + group.getId();
		String MY_SEARCHBASE = OrganizationLDAPConfig.USERS_URL;

		LDAPSearchResults res =
			service.search(
				MY_SEARCHBASE,
				LDAPConnection.SCOPE_SUB,
				MY_FILTER,
				null,
				false);

		LDAPEntry findEntry = (LDAPEntry) res.next();
		LDAPModificationSet mods = new LDAPModificationSet();
		LDAPAttribute attrMembership = new LDAPAttribute("membership");
		mods.add(LDAPModification.DELETE, attrMembership);
		service.modify(findEntry.getDN(), mods);
	}

	Collection findMembershipsByUser(String userName, Session session)
		throws Exception {
		List memberships = new ArrayList();
		LDAPService service = null;

		try {

			String MY_FILTER = "uid=" + userName;
			String MY_SEARCHBASE = USERSURL;

			LDAPSearchResults res =
				service.search(
					MY_SEARCHBASE,
					LDAPConnection.SCOPE_SUB,
					MY_FILTER,
					null,
					false);
			List l = getUtil(res);

			while (res.hasMoreElements()) {
				LDAPEntry findEntry = res.next();

				String[] attrValues =
					(String[]) getutil("membership",
						findEntry.getAttributeSet(),
						false)
						.toArray();
				for (int i = 0; i < attrValues.length; i++) {
					String[] membership = StringUtils.split(attrValues[i], ",");
					MembershipImpl mi = new MembershipImpl();
					mi.setId(attrValues[i]);
					mi.setUserName(membership[0]);
					mi.setMembershipType(membership[1]);					
					mi.setGroupId(membership[2]);
					memberships.add(mi);
				}
			}
		} finally {
			serviceContainer_.closeLDAPService(service);
		}
		return memberships;
	}

	public Collection findMembershipsByGroup(Group group) throws Exception {
		Collection memberships = null;

		LDAPService service = null;
		Session session = null;
		try {
			service = serviceContainer_.createLDAPService();
			session = hService_.openSession();
			String MY_FILTER = "membership=" + "*" + group;
			String MY_SEARCHBASE = USERSURL;

			LDAPSearchResults res =
				service.search(
					MY_SEARCHBASE,
					LDAPConnection.SCOPE_SUB,
					MY_FILTER,
					null,
					false);
			List l = getUtil(res);

			while (res.hasMoreElements()) {
				LDAPEntry findEntry = res.next();

				String[] attrValues =
					(String[]) getutil("membership",
						findEntry.getAttributeSet(),
						false)
						.toArray();
				for (int i = 0; i < attrValues.length; i++) {
					String[] membership = StringUtils.split(attrValues[i], ",");
					MembershipImpl mi = new MembershipImpl();
					mi.setId(attrValues[i]);
					mi.setUserName(membership[0]);
					mi.setMembershipType(membership[1]);					
					mi.setGroupId(membership[2]);
					memberships.add(mi);
				}
			}
		} finally {
			serviceContainer_.closeLDAPService(service);
			hService_.closeSession(session);
		}
		return memberships;

	}

	public Membership findMembership(String id) throws Exception {
		MembershipImpl membership = null;
		LDAPService service = null;

		try {

			String MY_FILTER = "membership=" + id;
			String MY_SEARCHBASE = USERSURL;

			LDAPSearchResults res =
				service.search(
					MY_SEARCHBASE,
					LDAPConnection.SCOPE_SUB,
					MY_FILTER,
					null,
					false);
			List l = getUtil(res);

			LDAPEntry findEntry = (LDAPEntry) l.get(0);
			String[] attrValues =
				(String[]) getutil("membership",
					findEntry.getAttributeSet(),
					false)
					.toArray();
			for (int i = 0; i < attrValues.length; i++) {
				if (id.equals(attrValues[i])) {
					String[] array = StringUtils.split(attrValues[i], ",");
					membership = new MembershipImpl();
					membership.setId(attrValues[i]);
					membership.setUserName(array[0]);
					membership.setMembershipType(array[1]);					
					membership.setGroupId(array[2]);

				}
			}
		} finally {
			serviceContainer_.closeLDAPService(service);
		}
		return membership;
	}

	private void preSave(Membership membership, boolean isNew, Session session)
		throws Exception {
		XResources xresources = new XResources();
		xresources.addResource(Session.class, session);
		for (int i = 0; i < listeners_.size(); i++) {
			MembershipEventListener listener =
				(MembershipEventListener) listeners_.get(i);
			listener.preSave(membership, isNew, xresources);
		}
	}

	private void postSave(
		Membership membership,
		boolean isNew,
		Session session)
		throws Exception {
		XResources xresources = new XResources();
		xresources.addResource(Session.class, session);
		for (int i = 0; i < listeners_.size(); i++) {
			MembershipEventListener listener =
				(MembershipEventListener) listeners_.get(i);
			listener.postSave(membership, isNew, xresources);
		}
	}

	private void preDelete(Membership membership, Session session)
		throws Exception {
		XResources xresources = new XResources();
		xresources.addResource(Session.class, session);
		for (int i = 0; i < listeners_.size(); i++) {
			MembershipEventListener listener =
				(MembershipEventListener) listeners_.get(i);
			listener.preDelete(membership, xresources);
		}
	}

	private void postDelete(Membership membership, Session session)
		throws Exception {
		XResources xresources = new XResources();
		xresources.addResource(Session.class, session);
		for (int i = 0; i < listeners_.size(); i++) {
			MembershipEventListener listener =
				(MembershipEventListener) listeners_.get(i);
			listener.postDelete(membership, xresources);
		}
	}
}