/**
 * Copyright 2001-2004 The eXo Platform SARL         All rights reserved.
 * Please look at license.txt in info directory for more license detail.
 **/
package org.exoplatform.services.organization.ldap;

import java.util.ArrayList;
import java.util.Collection;
import java.util.List;

import net.sf.hibernate.Session;
import netscape.ldap.LDAPAttribute;
import netscape.ldap.LDAPAttributeSet;
import netscape.ldap.LDAPCompareAttrNames;
import netscape.ldap.LDAPConnection;
import netscape.ldap.LDAPEntry;
import netscape.ldap.LDAPModification;
import netscape.ldap.LDAPModificationSet;
import netscape.ldap.LDAPSearchResults;

import org.exoplatform.commons.utils.ObjectPageList;
import org.exoplatform.commons.utils.PageList;
import org.exoplatform.services.database.HibernateService;
import org.exoplatform.services.database.XResources;
import org.exoplatform.services.ldap.LDAPService;
import org.exoplatform.services.ldap.LDAPServiceContainer;
import org.exoplatform.services.organization.Query;
import org.exoplatform.services.organization.User;
import org.exoplatform.services.organization.UserEventListener;
import org.exoplatform.services.organization.impl.UserImpl;
/**
 * Created by the eXo platform team
 * User: Daniel Summer
 * Date: 28 august 2004
 *
 */
public class UserLDAPHandler extends BaseLDAPHandler {

	private final String BASEURL = OrganizationLDAPConfig.BASE_URL;
	private final String PORTALURL = OrganizationLDAPConfig.PORTAL_URL;
	private final String ROLESURL = OrganizationLDAPConfig.ROLES_URL;
	private final String USERSURL = OrganizationLDAPConfig.USERS_URL;

	private LDAPServiceContainer serviceContainer_;
	private HibernateService hService_;
	private List listeners_;

	public UserLDAPHandler(
		LDAPServiceContainer serviceContainer,
		HibernateService hService) {
		serviceContainer_ = serviceContainer;
		hService_ = hService;
		listeners_ = new ArrayList(5);
	}

	final public List getUserEventListeners() {
		return listeners_;
	}

	public void addUserEventListener(UserEventListener listener) {
		listeners_.add(listener);
	}

	public void createUser(User user) throws Exception {
		LDAPService service = null;
		Session session = null;
		try {
			service = serviceContainer_.createLDAPService();
			session = hService_.openSession();
			String dn = "uid=" + user.getUserName() + "," + USERSURL;
			LDAPAttributeSet attrs = new LDAPAttributeSet();

			String objectclass_values[] = { "exouser" };

			setutil("objectclass", objectclass_values, attrs, false);
			setutil("uid", user.getUserName(), attrs);
			setutil("userPassword", user.getPassword(), attrs);
			setutil("firstname", user.getFirstName(), attrs);
			setutil("lastname", user.getLastName(), attrs);
			setutil("mail", user.getEmail(), attrs);
			UserProfileLDAPData upd =
				new UserProfileLDAPData(user.getUserName());
			setutil("profile", upd.getProfile(), attrs);
			LDAPEntry myEntry = new LDAPEntry(dn, attrs);
			preSave(user, true, session);
			service.add(myEntry);
			postSave(user, true, session);
			session.flush();
		} finally {
			serviceContainer_.closeLDAPService(service);
			hService_.closeSession(session);
		}
	}

	public void saveUser(User user) throws Exception {
		LDAPService service = null;
		Session session = null;
		try {
			service = serviceContainer_.createLDAPService();
			session = hService_.openSession();
			String dn = "uid=" + user.getUserName() + "," + USERSURL;
			LDAPModificationSet mods = new LDAPModificationSet();
			mods.add(
				LDAPModification.REPLACE,
				new LDAPAttribute("userPassword", user.getPassword()));
			mods.add(
				LDAPModification.REPLACE,
				new LDAPAttribute("firstname", user.getFirstName()));
			mods.add(
				LDAPModification.REPLACE,
				new LDAPAttribute("lastname", user.getLastName()));
			mods.add(
				LDAPModification.REPLACE,
				new LDAPAttribute("mail", user.getEmail()));

			preSave(user, false, session);
			service.modify(dn, mods);
			postSave(user, false, session);
			session.flush();
		} finally {
			serviceContainer_.closeLDAPService(service);
			hService_.closeSession(session);
		}
	}

	void createUserEntry(User user, Session session) throws Exception {
		LDAPService service = null;

		try {
			service = serviceContainer_.createLDAPService();

			String dn = "uid=" + user.getUserName() + "," + USERSURL;
			LDAPAttributeSet attrs = new LDAPAttributeSet();

			String objectclass_values[] = { "exouser" };

			setutil("objectclass", objectclass_values, attrs, false);
			setutil("uid", user.getUserName(), attrs);
			setutil("userPassword", user.getPassword(), attrs);
			setutil("firstname", user.getFirstName(), attrs);
			setutil("lastname", user.getLastName(), attrs);
			setutil("mail", user.getEmail(), attrs);
			UserProfileLDAPData upd =
				new UserProfileLDAPData(user.getUserName());
			setutil("profile", upd.getProfile(), attrs);
			LDAPEntry myEntry = new LDAPEntry(dn, attrs);

			service.add(myEntry);

		} finally {
			serviceContainer_.closeLDAPService(service);

		}
	}

	public User removeUser(String userName) throws Exception {
		LDAPService service = null;
		Session session = null;
		User foundUser = null;
		try {
			service = serviceContainer_.createLDAPService();
			session = hService_.openSession();
			String MY_FILTER = "uid=" + userName + ",";
			String MY_SEARCHBASE = USERSURL;

			String ENTRYDN = MY_FILTER + MY_SEARCHBASE;

			foundUser = findUserByName(userName);
			preDelete(foundUser, session);
			MembershipLDAPHandler.removeMembershipEntriesOfUser(userName,	session,service);
			UserProfileLDAPHandler.removeUserProfileEntry(userName,	session, service);
			service.delete(ENTRYDN);

			postDelete(foundUser, session);
			session.flush();
		} finally {
			hService_.closeSession(session);
			serviceContainer_.closeLDAPService(service);
		}
		return foundUser;
	}

	/*This method use with importer/exporter */
	void removeUserEntry(User user, Session session) throws Exception {
		LDAPService service = null;

		service = serviceContainer_.createLDAPService();

		String MY_FILTER = "uid=" + user.getUserName();
		String MY_SEARCHBASE = USERSURL;

		String ENTRYDN = MY_FILTER + MY_SEARCHBASE;

		service.delete(ENTRYDN);
		serviceContainer_.closeLDAPService(service);

	}

	public User findUserByName(String userName) throws Exception {
		User user = null;
		LDAPService service = null;
		try {
			service = serviceContainer_.createLDAPService();
			user = findUserByName(userName, service);
		} finally {
			serviceContainer_.closeLDAPService(service);
		}
		return user;
	}

	public User findUserByName(String userName, LDAPService service)
		throws Exception {
		User user = new UserImpl();
		String MY_FILTER = "uid=" + userName;
		String MY_SEARCHBASE = USERSURL;

		LDAPSearchResults res =
			service.search(	MY_SEARCHBASE,LDAPConnection.SCOPE_SUB,	MY_FILTER,	null,	false);

		List l = getUtil(res);
		LDAPEntry findEntry = (LDAPEntry) l.get(0);

		LDAPAttributeSet attrs = findEntry.getAttributeSet();
		user.setUserName(getutil("uid", attrs));
		user.setPassword(getutil("userPassword", attrs));
		user.setFirstName(getutil("firstname", attrs));
		user.setLastName(getutil("lastname", attrs));
		user.setEmail(getutil("mail", attrs));

		return user;
	}

	public PageList getUserPageList(int pageSize) throws Exception {
		LDAPService service = null;
		Session session = null;
		service = serviceContainer_.createLDAPService();
		session = hService_.openSession();

		String MY_SEARCHBASE = USERSURL;

		LDAPSearchResults res =
			service.search(
				MY_SEARCHBASE,
				LDAPConnection.SCOPE_SUB,
				null,
				null,
				false);
		String[] sortAttrs = { "uid" };
		boolean[] ascending = { true, true };
		res.sort(new LDAPCompareAttrNames(sortAttrs, ascending));

		List users = new ArrayList();
		while (res.hasMoreElements()) {
			LDAPEntry findEntry = null;
			findEntry = res.next();
			LDAPAttributeSet attrs = findEntry.getAttributeSet();
			User user = new UserImpl();
			user.setUserName(getutil("uid", attrs));
			user.setPassword(getutil("userPassword", attrs));
			user.setFirstName(getutil("firstname", attrs));
			user.setLastName(getutil("lastname", attrs));
			user.setEmail(getutil("mail", attrs));
			users.add(user);
		}
		hService_.closeSession(session);
		serviceContainer_.closeLDAPService(service);
		//return users;
		return new ObjectPageList(users, pageSize);
		//return new DBObjectPageList(service_, UserImpl.class, pageSize);
	}

	public PageList findUsers(Query q) throws Exception {

		LDAPService service = null;
		Session session = null;
		service = serviceContainer_.createLDAPService();
		session = hService_.openSession();
		String MY_FILTER = null;
		if (q.getUserName() != null) {
			String userName = q.getUserName();
			if (userName == null || userName.length() == 0) {
				userName = "%";
			}
			userName = userName.replace('*', '%');
			MY_FILTER = "uid ~= " + userName;
		}

		String MY_SEARCHBASE = USERSURL;

		LDAPSearchResults res =
			service.search(
				MY_SEARCHBASE,
				LDAPConnection.SCOPE_SUB,
				MY_FILTER,
				null,
				false);
		String[] sortAttrs = { "uid" };
		boolean[] ascending = { true, true };
		res.sort(new LDAPCompareAttrNames(sortAttrs, ascending));

		List users = new ArrayList();
		while (res.hasMoreElements()) {
			LDAPEntry findEntry = null;
			findEntry = res.next();
			LDAPAttributeSet attrs = findEntry.getAttributeSet();
			User user = new UserImpl();
			user.setUserName(getutil("uid", attrs));
			user.setPassword(getutil("userPassword", attrs));
			user.setFirstName(getutil("firstname", attrs));
			user.setLastName(getutil("lastname", attrs));
			user.setEmail(getutil("mail", attrs));
			users.add(user);
		}
		/*
				int first = 0;
				int max = 0;
				int size = users.size();
				int from = 0, to = 0;
				if (q.getMaxSize() > 0) {
					first = q.getFrom();
					max = q.getTo();
					from = (first > size) ? 0 : first;
					if (max > size) {
						to = (size > 0) ? size : 0;
					} else {
						to = first + max - 1;
					}
		
				}
				//users.subList(from, to);
		*/
		hService_.closeSession(session);
		serviceContainer_.closeLDAPService(service);
		//return users;
		return new ObjectPageList(users, 20);
	}

	public PageList findUsersByGroup(String groupId) throws Exception {

		LDAPService service = null;
		Session session = null;
		service = serviceContainer_.createLDAPService();
		session = hService_.openSession();

		String MY_SEARCHBASE = USERSURL;
		String MY_FILTER = "membership=" + "*" + groupId;
		LDAPSearchResults res =
			service.search(
				MY_SEARCHBASE,
				LDAPConnection.SCOPE_SUB,
				MY_FILTER,
				null,
				false);
		String[] sortAttrs = { "uid" };
		boolean[] ascending = { true, true };
		res.sort(new LDAPCompareAttrNames(sortAttrs, ascending));

		List users = new ArrayList();

		while (res.hasMoreElements()) {
			LDAPEntry findEntry = null;
			findEntry = res.next();
			LDAPAttributeSet attrs = findEntry.getAttributeSet();

			User user = new UserImpl();
			user.setUserName(getutil("uid", attrs));
			user.setPassword(getutil("userPassword", attrs));
			user.setFirstName(getutil("firstname", attrs));
			user.setLastName(getutil("lastname", attrs));
			user.setEmail(getutil("mail", attrs));
			users.add(user);

		}

		hService_.closeSession(session);
		serviceContainer_.closeLDAPService(service);
		//return users;
		return new ObjectPageList(users, 20);
	}

	public Collection findUsersByGroupAndRole(String groupName, String role)
		throws Exception {

		LDAPService service = null;
		Session session = null;
		service = serviceContainer_.createLDAPService();
		session = hService_.openSession();

		String MY_SEARCHBASE = USERSURL;
		String FILTER =
			"(&(membership=" + "*" + groupName + ")(exorole=" + role + "))";

		LDAPSearchResults res =
			service.search(
				MY_SEARCHBASE,
				LDAPConnection.SCOPE_SUB,
				FILTER,
				null,
				false);
		String[] sortAttrs = { "uid" };
		boolean[] ascending = { true, true };
		res.sort(new LDAPCompareAttrNames(sortAttrs, ascending));

		List users = new ArrayList();

		while (res.hasMoreElements()) {
			LDAPEntry findEntry = null;
			findEntry = res.next();
			LDAPAttributeSet attrs = findEntry.getAttributeSet();

			User user = new UserImpl();
			user.setUserName(getutil("uid", attrs));
			user.setPassword(getutil("userPassword", attrs));
			user.setFirstName(getutil("firstname", attrs));
			user.setLastName(getutil("lastname", attrs));
			user.setEmail(getutil("mail", attrs));
			users.add(user);

		}

		hService_.closeSession(session);
		serviceContainer_.closeLDAPService(service);
		return users;
	}

	private void preSave(User user, boolean isNew, Session session)
		throws Exception {
		XResources xresources = new XResources();
		xresources.addResource(Session.class, session);
		for (int i = 0; i < listeners_.size(); i++) {
			UserEventListener listener = (UserEventListener) listeners_.get(i);
			listener.preSave(user, isNew, xresources);
		}
	}

	private void postSave(User user, boolean isNew, Session session)
		throws Exception {
		XResources xresources = new XResources();
		xresources.addResource(Session.class, session);
		for (int i = 0; i < listeners_.size(); i++) {
			UserEventListener listener = (UserEventListener) listeners_.get(i);
			listener.postSave(user, isNew, xresources);
		}
	}

	private void preDelete(User user, Session session) throws Exception {
		XResources xresources = new XResources();
		xresources.addResource(Session.class, session);
		for (int i = 0; i < listeners_.size(); i++) {
			UserEventListener listener = (UserEventListener) listeners_.get(i);
			listener.preDelete(user, xresources);
		}
	}

	private void postDelete(User user, Session session) throws Exception {
		XResources xresources = new XResources();
		xresources.addResource(Session.class, session);
		for (int i = 0; i < listeners_.size(); i++) {
			UserEventListener listener = (UserEventListener) listeners_.get(i);
			listener.postDelete(user, xresources);
		}
	}
}