/**
 * Copyright 2001-2004 The eXo Platform SARL         All rights reserved.
 * Please look at license.txt in info directory for more license detail.
 **/
package org.exoplatform.services.organization.ldap;


import java.util.ArrayList;
import java.util.Collection;
import java.util.List;

import net.sf.hibernate.Session;
import netscape.ldap.LDAPAttribute;
import netscape.ldap.LDAPAttributeSet;
import netscape.ldap.LDAPConnection;
import netscape.ldap.LDAPEntry;
import netscape.ldap.LDAPException;
import netscape.ldap.LDAPModification;
import netscape.ldap.LDAPModificationSet;
import netscape.ldap.LDAPSearchResults;

import org.exoplatform.services.database.HibernateService;
import org.exoplatform.services.ldap.LDAPService;
import org.exoplatform.services.ldap.LDAPServiceContainer;
import org.exoplatform.services.organization.UserProfile;
import org.exoplatform.services.organization.UserProfileEventListener;

/**
 * Created by the eXo platform team
 * User: Daniel Summer
 * Date: 28 august 2004
 *
 */
public class UserProfileLDAPHandler extends BaseLDAPHandler {
	private final String BASEURL = OrganizationLDAPConfig.BASE_URL;
	private final String PORTALURL = OrganizationLDAPConfig.PORTAL_URL;
	private final String ROLESURL = OrganizationLDAPConfig.ROLES_URL;
	private final String USERSURL = OrganizationLDAPConfig.USERS_URL;

	private LDAPServiceContainer serviceContainer_;
	private HibernateService service_;
	private List listeners_;

	public UserProfileLDAPHandler(
		LDAPServiceContainer serviceContainer,
		HibernateService service) {
		serviceContainer_ = serviceContainer;
		service_ = service;
		listeners_ = new ArrayList(5);
	}

	public void addUserProfileEventListener(UserProfileEventListener listener) {
		listeners_.add(listener);
	}

	void createUserProfileEntry(UserProfile up, Session session)
		throws Exception {
		UserProfileLDAPData upd = new UserProfileLDAPData();
		LDAPService service = null;
		service = serviceContainer_.createLDAPService();
		String MY_FILTER = "uid=" + up.getUserName();
		String MY_SEARCHBASE = USERSURL;
		LDAPSearchResults res =
			service.search(
				MY_SEARCHBASE,
				LDAPConnection.SCOPE_SUB,
				MY_FILTER,
				null,
				false);
		List l = getUtil(res);
		LDAPEntry findEntry = (LDAPEntry) l.get(0);
		upd.setUserProfile(up);
		LDAPModificationSet mods = new LDAPModificationSet();
		mods.add(
			LDAPModification.REPLACE,
			new LDAPAttribute("profile", upd.getProfile()));
		service.modify(findEntry.getDN(), mods);
		serviceContainer_.closeLDAPService(service);
	}

	public void saveUserProfile(UserProfile profile) throws Exception {
		LDAPService service = null;

		UserProfileLDAPData upd = null;
		try {
			service = serviceContainer_.createLDAPService();
			String MY_FILTER = "uid=" + profile.getUserName();
			String MY_SEARCHBASE = USERSURL;
			LDAPSearchResults res =
				service.search(
					MY_SEARCHBASE,
					LDAPConnection.SCOPE_SUB,
					MY_FILTER,
					null,
					false);
			List l = getUtil(res);
			LDAPEntry findEntry = (LDAPEntry) l.get(0);
			upd = new UserProfileLDAPData();
			upd.setUserProfile(profile);
			LDAPModificationSet mods = new LDAPModificationSet();
			mods.add(
				LDAPModification.REPLACE,
				new LDAPAttribute("profile", upd.getProfile()));
			service.modify(findEntry.getDN(), mods);
		} finally {
			serviceContainer_.closeLDAPService(service);
		}
	}

	public UserProfile removeUserProfile(String userName) throws Exception {
		LDAPService service = null;
		UserProfileLDAPData upd = null;
		try {
			service = serviceContainer_.createLDAPService();
			String MY_FILTER = "uid=" + userName;
			String MY_SEARCHBASE = USERSURL;
			LDAPSearchResults res =
				service.search(
					MY_SEARCHBASE,
					LDAPConnection.SCOPE_SUB,
					MY_FILTER,
					null,
					false);
			List l = getUtil(res);
			LDAPEntry findEntry = (LDAPEntry) l.get(0);
			LDAPModificationSet mods = new LDAPModificationSet();
			LDAPAttribute attrRole = new LDAPAttribute("role");
			mods.add(LDAPModification.DELETE, attrRole);
			service.modify(findEntry.getDN(), mods);
		} finally {
			service = serviceContainer_.createLDAPService();
		}
		return upd.getUserProfile();
	}

	public UserProfile findUserProfileByName(String userName)
		throws Exception {
		LDAPService service = null;
		try {
			service = serviceContainer_.createLDAPService();
			return findUserProfileByName(userName, service);
		} finally {
			serviceContainer_.closeLDAPService(service);
		}
	}

	public UserProfile findUserProfileByName(
		String userName,
		LDAPService service)
		throws Exception {
		UserProfileLDAPData upd = new UserProfileLDAPData();
		String MY_FILTER = "uid=" + userName;
		String MY_SEARCHBASE = USERSURL;
		LDAPSearchResults res =
			service.search(
				MY_SEARCHBASE,
				LDAPConnection.SCOPE_SUB,
				MY_FILTER,
				null,
				false);
		List l = getUtil(res);
		LDAPEntry findEntry = (LDAPEntry) l.get(0);
		LDAPAttributeSet attrs = findEntry.getAttributeSet();
		upd.setProfile(getutil("profile", attrs));
		upd.setUserName(userName);

		if (upd != null) {
			return upd.getUserProfile();
		}
		return null;
	}

	static void removeUserProfileEntry(
		String userName,
		Session session,
		LDAPService service)
		throws Exception {

		UserProfileLDAPData upd = null;

		String MY_FILTER = "uid=" + userName;
		String MY_SEARCHBASE = OrganizationLDAPConfig.USERS_URL;;
		LDAPSearchResults res =
			service.search(
				MY_SEARCHBASE,
				LDAPConnection.SCOPE_SUB,
				MY_FILTER,
				null,
				false);
		while (res.hasMoreElements()) {

			LDAPEntry findEntry = null;
			try {
				findEntry = res.next();
				LDAPModificationSet mods = new LDAPModificationSet();
				LDAPAttribute attrRole = new LDAPAttribute("profile");
				mods.add(LDAPModification.DELETE, attrRole);
				service.modify(findEntry.getDN(), mods);

			} catch (LDAPException e) {
				continue;
			}
		}

	}

	public Collection findUserProfiles() throws Exception {
		return null;
	}
}